@@ -6,8 +6,15 @@ coredis is an async redis client with support for redis server,
 cluster & sentinel.
 """
 
+from __future__ import annotations
+
 from coredis.client import Redis, RedisCluster
-from coredis.connection import ClusterConnection, Connection, UnixDomainSocketConnection
+from coredis.connection import (
+    BaseConnection,
+    ClusterConnection,
+    Connection,
+    UnixDomainSocketConnection,
+)
 from coredis.exceptions import (
     AskError,
     AuthenticationError,
@@ -18,34 +25,42 @@ from coredis.exceptions import (
     ClusterCrossSlotError,
     ClusterDownError,
     ClusterError,
+    ClusterResponseError,
     ClusterTransactionError,
+    CommandNotSupportedError,
     CommandSyntaxError,
     ConnectionError,
     DataError,
     ExecAbortError,
+    FunctionError,
     InvalidResponse,
     LockError,
     MovedError,
     NoKeyError,
     NoScriptError,
+    PrimaryNotFoundError,
+    ProtocolError,
     PubSubError,
     ReadOnlyError,
     RedisClusterException,
     RedisError,
+    ReplicaNotFoundError,
     ResponseError,
     TimeoutError,
     TryAgainError,
+    UnknownCommandError,
     WatchError,
 )
+from coredis.nodemanager import Node, NodeFlag
 from coredis.pool import BlockingConnectionPool, ClusterConnectionPool, ConnectionPool
 from coredis.tokens import PureToken
-from coredis.utils import NodeFlag
 
 from . import _version
 
 __all__ = [
     "Redis",
     "RedisCluster",
+    "BaseConnection",
     "Connection",
     "UnixDomainSocketConnection",
     "ClusterConnection",
@@ -58,28 +73,36 @@ __all__ = [
     "AuthenticationRequiredError",
     "AuthorizationError",
     "BusyLoadingError",
+    "CommandNotSupportedError",
     "ClusterCrossSlotError",
     "ClusterDownError",
     "ClusterError",
+    "ClusterResponseError",
     "ClusterTransactionError",
     "CommandSyntaxError",
     "ConnectionError",
     "DataError",
     "ExecAbortError",
+    "FunctionError",
     "InvalidResponse",
     "LockError",
     "MovedError",
+    "Node",
     "NodeFlag",
     "NoKeyError",
     "NoScriptError",
+    "PrimaryNotFoundError",
+    "ProtocolError",
     "PubSubError",
     "PureToken",
     "ReadOnlyError",
     "RedisClusterException",
     "RedisError",
+    "ReplicaNotFoundError",
     "ResponseError",
     "TimeoutError",
     "TryAgainError",
+    "UnknownCommandError",
     "WatchError",
 ]
 
@@ -88,4 +111,4 @@ StrictRedis = Redis
 StrictRedisCluster = RedisCluster
 
 
-__version__ = _version.get_versions()["version"]
+__version__ = _version.get_versions()["version"]  # type: ignore
