@@ -1,7 +1,9 @@
-import enum
+from __future__ import annotations
+
+from coredis._utils import CaseAndEncodingInsensitiveEnum
 
 
-class PureToken(str, enum.Enum):
+class PureToken(CaseAndEncodingInsensitiveEnum):
     """
     Enum for using pure-tokens with the redis api.
     """
@@ -9,39 +11,39 @@ class PureToken(str, enum.Enum):
     #: Used by:
     #:
     #:  - ``ACL LOG``
-    RESET = "RESET"
+    RESET = b"RESET"
 
     #: Used by:
     #:
     #:  - ``BGSAVE``
-    SCHEDULE = "SCHEDULE"
+    SCHEDULE = b"SCHEDULE"
 
     #: Used by:
     #:
     #:  - ``BITCOUNT``
     #:  - ``BITPOS``
-    BIT = "BIT"
+    BIT = b"BIT"
 
     #: Used by:
     #:
     #:  - ``BITCOUNT``
     #:  - ``BITPOS``
-    BYTE = "BYTE"
+    BYTE = b"BYTE"
 
     #: Used by:
     #:
     #:  - ``BITFIELD``
-    FAIL = "FAIL"
+    FAIL = b"FAIL"
 
     #: Used by:
     #:
     #:  - ``BITFIELD``
-    SAT = "SAT"
+    SAT = b"SAT"
 
     #: Used by:
     #:
     #:  - ``BITFIELD``
-    WRAP = "WRAP"
+    WRAP = b"WRAP"
 
     #: Used by:
     #:
@@ -49,7 +51,7 @@ class PureToken(str, enum.Enum):
     #:  - ``BLMPOP``
     #:  - ``LMOVE``
     #:  - ``LMPOP``
-    LEFT = "LEFT"
+    LEFT = b"LEFT"
 
     #: Used by:
     #:
@@ -57,7 +59,7 @@ class PureToken(str, enum.Enum):
     #:  - ``BLMPOP``
     #:  - ``LMOVE``
     #:  - ``LMPOP``
-    RIGHT = "RIGHT"
+    RIGHT = b"RIGHT"
 
     #: Used by:
     #:
@@ -67,7 +69,7 @@ class PureToken(str, enum.Enum):
     #:  - ``ZMPOP``
     #:  - ``ZUNION``
     #:  - ``ZUNIONSTORE``
-    MAX = "MAX"
+    MAX = b"MAX"
 
     #: Used by:
     #:
@@ -77,107 +79,107 @@ class PureToken(str, enum.Enum):
     #:  - ``ZMPOP``
     #:  - ``ZUNION``
     #:  - ``ZUNIONSTORE``
-    MIN = "MIN"
+    MIN = b"MIN"
 
     #: Used by:
     #:
     #:  - ``CLIENT CACHING``
     #:  - ``SCRIPT DEBUG``
-    NO = "NO"
+    NO = b"NO"
 
     #: Used by:
     #:
     #:  - ``CLIENT CACHING``
     #:  - ``SCRIPT DEBUG``
-    YES = "YES"
+    YES = b"YES"
 
     #: Used by:
     #:
     #:  - ``CLIENT KILL``
     #:  - ``CLIENT LIST``
-    MASTER = "MASTER"
+    MASTER = b"MASTER"
 
     #: Used by:
     #:
     #:  - ``CLIENT KILL``
     #:  - ``CLIENT LIST``
-    NORMAL = "NORMAL"
+    NORMAL = b"NORMAL"
 
     #: Used by:
     #:
     #:  - ``CLIENT KILL``
     #:  - ``CLIENT LIST``
-    PUBSUB = "PUBSUB"
+    PUBSUB = b"PUBSUB"
 
     #: Used by:
     #:
     #:  - ``CLIENT KILL``
     #:  - ``CLIENT LIST``
-    REPLICA = "REPLICA"
+    REPLICA = b"REPLICA"
 
     #: Used by:
     #:
     #:  - ``CLIENT KILL``
-    SLAVE = "SLAVE"
+    SLAVE = b"SLAVE"
 
     #: Used by:
     #:
     #:  - ``CLIENT NO-EVICT``
     #:  - ``CLIENT REPLY``
     #:  - ``CLIENT TRACKING``
-    OFF = "OFF"
+    OFF = b"OFF"
 
     #: Used by:
     #:
     #:  - ``CLIENT NO-EVICT``
     #:  - ``CLIENT REPLY``
     #:  - ``CLIENT TRACKING``
-    ON = "ON"
+    ON = b"ON"
 
     #: Used by:
     #:
     #:  - ``CLIENT PAUSE``
-    ALL = "ALL"
+    ALL = b"ALL"
 
     #: Used by:
     #:
     #:  - ``CLIENT PAUSE``
-    WRITE = "WRITE"
+    WRITE = b"WRITE"
 
     #: Used by:
     #:
     #:  - ``CLIENT REPLY``
-    SKIP = "SKIP"
+    SKIP = b"SKIP"
 
     #: Used by:
     #:
     #:  - ``CLIENT TRACKING``
-    BCAST = "BCAST"
+    BCAST = b"BCAST"
 
     #: Used by:
     #:
     #:  - ``CLIENT TRACKING``
-    NOLOOP = "NOLOOP"
+    NOLOOP = b"NOLOOP"
 
     #: Used by:
     #:
     #:  - ``CLIENT TRACKING``
-    OPTIN = "OPTIN"
+    OPTIN = b"OPTIN"
 
     #: Used by:
     #:
     #:  - ``CLIENT TRACKING``
-    OPTOUT = "OPTOUT"
+    OPTOUT = b"OPTOUT"
 
     #: Used by:
     #:
     #:  - ``CLIENT UNBLOCK``
-    ERROR = "ERROR"
+    ERROR = b"ERROR"
 
     #: Used by:
     #:
     #:  - ``CLIENT UNBLOCK``
-    TIMEOUT = "TIMEOUT"
+    TIMEOUT = b"TIMEOUT"
 
     #: Used by:
     #:
@@ -185,27 +187,27 @@ class PureToken(str, enum.Enum):
     #:  - ``FAILOVER``
     #:  - ``SHUTDOWN``
     #:  - ``XCLAIM``
-    FORCE = "FORCE"
+    FORCE = b"FORCE"
 
     #: Used by:
     #:
     #:  - ``CLUSTER FAILOVER``
-    TAKEOVER = "TAKEOVER"
+    TAKEOVER = b"TAKEOVER"
 
     #: Used by:
     #:
     #:  - ``CLUSTER RESET``
-    HARD = "HARD"
+    HARD = b"HARD"
 
     #: Used by:
     #:
     #:  - ``CLUSTER RESET``
-    SOFT = "SOFT"
+    SOFT = b"SOFT"
 
     #: Used by:
     #:
     #:  - ``CLUSTER SETSLOT``
-    STABLE = "STABLE"
+    STABLE = b"STABLE"
 
     #: Used by:
     #:
@@ -214,7 +216,7 @@ class PureToken(str, enum.Enum):
     #:  - ``FUNCTION RESTORE``
     #:  - ``MIGRATE``
     #:  - ``RESTORE``
-    REPLACE = "REPLACE"
+    REPLACE = b"REPLACE"
 
     #: Used by:
     #:
@@ -223,7 +225,7 @@ class PureToken(str, enum.Enum):
     #:  - ``PEXPIRE``
     #:  - ``PEXPIREAT``
     #:  - ``ZADD``
-    GT = "GT"
+    GT = b"GT"
 
     #: Used by:
     #:
@@ -232,7 +234,7 @@ class PureToken(str, enum.Enum):
     #:  - ``PEXPIRE``
     #:  - ``PEXPIREAT``
     #:  - ``ZADD``
-    LT = "LT"
+    LT = b"LT"
 
     #: Used by:
     #:
@@ -243,7 +245,7 @@ class PureToken(str, enum.Enum):
     #:  - ``PEXPIREAT``
     #:  - ``SET``
     #:  - ``ZADD``
-    NX = "NX"
+    NX = b"NX"
 
     #: Used by:
     #:
@@ -254,13 +256,13 @@ class PureToken(str, enum.Enum):
     #:  - ``PEXPIREAT``
     #:  - ``SET``
     #:  - ``ZADD``
-    XX = "XX"
+    XX = b"XX"
 
     #: Used by:
     #:
     #:  - ``FAILOVER``
     #:  - ``SHUTDOWN``
-    ABORT = "ABORT"
+    ABORT = b"ABORT"
 
     #: Used by:
     #:
@@ -268,7 +270,7 @@ class PureToken(str, enum.Enum):
     #:  - ``FLUSHDB``
     #:  - ``FUNCTION FLUSH``
     #:  - ``SCRIPT FLUSH``
-    ASYNC = "ASYNC"
+    ASYNC = b"ASYNC"
 
     #: Used by:
     #:
@@ -277,28 +279,28 @@ class PureToken(str, enum.Enum):
     #:  - ``FUNCTION FLUSH``
     #:  - ``SCRIPT DEBUG``
     #:  - ``SCRIPT FLUSH``
-    SYNC = "SYNC"
+    SYNC = b"SYNC"
 
     #: Used by:
     #:
     #:  - ``FUNCTION LIST``
-    WITHCODE = "WITHCODE"
+    WITHCODE = b"WITHCODE"
 
     #: Used by:
     #:
     #:  - ``FUNCTION RESTORE``
-    APPEND = "APPEND"
+    APPEND = b"APPEND"
 
     #: Used by:
     #:
     #:  - ``FUNCTION RESTORE``
-    FLUSH = "FLUSH"
+    FLUSH = b"FLUSH"
 
     #: Used by:
     #:
     #:  - ``GEOADD``
     #:  - ``ZADD``
-    CHANGE = "CH"
+    CHANGE = b"CH"
 
     #: Used by:
     #:
@@ -309,7 +311,7 @@ class PureToken(str, enum.Enum):
     #:  - ``GEORADIUS_RO``
     #:  - ``GEOSEARCH``
     #:  - ``GEOSEARCHSTORE``
-    FT = "FT"
+    FT = b"FT"
 
     #: Used by:
     #:
@@ -320,7 +322,7 @@ class PureToken(str, enum.Enum):
     #:  - ``GEORADIUS_RO``
     #:  - ``GEOSEARCH``
     #:  - ``GEOSEARCHSTORE``
-    KM = "KM"
+    KM = b"KM"
 
     #: Used by:
     #:
@@ -331,7 +333,7 @@ class PureToken(str, enum.Enum):
     #:  - ``GEORADIUS_RO``
     #:  - ``GEOSEARCH``
     #:  - ``GEOSEARCHSTORE``
-    M = "M"
+    M = b"M"
 
     #: Used by:
     #:
@@ -342,7 +344,7 @@ class PureToken(str, enum.Enum):
     #:  - ``GEORADIUS_RO``
     #:  - ``GEOSEARCH``
     #:  - ``GEOSEARCHSTORE``
-    MI = "MI"
+    MI = b"MI"
 
     #: Used by:
     #:
@@ -352,7 +354,7 @@ class PureToken(str, enum.Enum):
     #:  - ``GEORADIUS_RO``
     #:  - ``GEOSEARCH``
     #:  - ``GEOSEARCHSTORE``
-    ANY = "ANY"
+    ANY = b"ANY"
 
     #: Used by:
     #:
@@ -364,7 +366,7 @@ class PureToken(str, enum.Enum):
     #:  - ``GEOSEARCHSTORE``
     #:  - ``SORT``
     #:  - ``SORT_RO``
-    ASC = "ASC"
+    ASC = b"ASC"
 
     #: Used by:
     #:
@@ -376,7 +378,7 @@ class PureToken(str, enum.Enum):
     #:  - ``GEOSEARCHSTORE``
     #:  - ``SORT``
     #:  - ``SORT_RO``
-    DESC = "DESC"
+    DESC = b"DESC"
 
     #: Used by:
     #:
@@ -385,7 +387,7 @@ class PureToken(str, enum.Enum):
     #:  - ``GEORADIUSBYMEMBER_RO``
     #:  - ``GEORADIUS_RO``
     #:  - ``GEOSEARCH``
-    WITHCOORD = "WITHCOORD"
+    WITHCOORD = b"WITHCOORD"
 
     #: Used by:
     #:
@@ -394,7 +396,7 @@ class PureToken(str, enum.Enum):
     #:  - ``GEORADIUSBYMEMBER_RO``
     #:  - ``GEORADIUS_RO``
     #:  - ``GEOSEARCH``
-    WITHDIST = "WITHDIST"
+    WITHDIST = b"WITHDIST"
 
     #: Used by:
     #:
@@ -403,154 +405,154 @@ class PureToken(str, enum.Enum):
     #:  - ``GEORADIUSBYMEMBER_RO``
     #:  - ``GEORADIUS_RO``
     #:  - ``GEOSEARCH``
-    WITHHASH = "WITHHASH"
+    WITHHASH = b"WITHHASH"
 
     #: Used by:
     #:
     #:  - ``GEOSEARCHSTORE``
-    STOREDIST = "STOREDIST"
+    STOREDIST = b"STOREDIST"
 
     #: Used by:
     #:
     #:  - ``GETEX``
-    PERSIST = "PERSIST"
+    PERSIST = b"PERSIST"
 
     #: Used by:
     #:
     #:  - ``HRANDFIELD``
-    WITHVALUES = "WITHVALUES"
+    WITHVALUES = b"WITHVALUES"
 
     #: Used by:
     #:
     #:  - ``LCS``
-    IDX = "IDX"
+    IDX = b"IDX"
 
     #: Used by:
     #:
     #:  - ``LCS``
-    LEN = "LEN"
+    LEN = b"LEN"
 
     #: Used by:
     #:
     #:  - ``LCS``
-    WITHMATCHLEN = "WITHMATCHLEN"
+    WITHMATCHLEN = b"WITHMATCHLEN"
 
     #: Used by:
     #:
     #:  - ``LINSERT``
-    AFTER = "AFTER"
+    AFTER = b"AFTER"
 
     #: Used by:
     #:
     #:  - ``LINSERT``
-    BEFORE = "BEFORE"
+    BEFORE = b"BEFORE"
 
     #: Used by:
     #:
     #:  - ``MIGRATE``
-    COPY = "COPY"
+    COPY = b"COPY"
 
     #: Used by:
     #:
     #:  - ``MIGRATE``
-    EMPTY_STRING = ""
+    EMPTY_STRING = b""
 
     #: Used by:
     #:
     #:  - ``RESTORE``
-    ABSTTL = "ABSTTL"
+    ABSTTL = b"ABSTTL"
 
     #: Used by:
     #:
     #:  - ``SET``
-    GET = "GET"
+    GET = b"GET"
 
     #: Used by:
     #:
     #:  - ``SET``
-    KEEPTTL = "KEEPTTL"
+    KEEPTTL = b"KEEPTTL"
 
     #: Used by:
     #:
     #:  - ``SHUTDOWN``
-    NOSAVE = "NOSAVE"
+    NOSAVE = b"NOSAVE"
 
     #: Used by:
     #:
     #:  - ``SHUTDOWN``
-    NOW = "NOW"
+    NOW = b"NOW"
 
     #: Used by:
     #:
     #:  - ``SHUTDOWN``
-    SAVE = "SAVE"
+    SAVE = b"SAVE"
 
     #: Used by:
     #:
     #:  - ``SORT``
     #:  - ``SORT_RO``
-    SORTING = "ALPHA"
+    SORTING = b"ALPHA"
 
     #: Used by:
     #:
     #:  - ``XADD``
     #:  - ``XTRIM``
-    APPROXIMATELY = "~"
+    APPROXIMATELY = b"~"
 
     #: Used by:
     #:
     #:  - ``XADD``
-    AUTO_ID = "*"
+    AUTO_ID = b"*"
 
     #: Used by:
     #:
     #:  - ``XADD``
     #:  - ``XTRIM``
-    EQUAL = "="
+    EQUAL = b"="
 
     #: Used by:
     #:
     #:  - ``XADD``
     #:  - ``XTRIM``
-    MAXLEN = "MAXLEN"
+    MAXLEN = b"MAXLEN"
 
     #: Used by:
     #:
     #:  - ``XADD``
     #:  - ``XTRIM``
-    MINID = "MINID"
+    MINID = b"MINID"
 
     #: Used by:
     #:
     #:  - ``XADD``
-    NOMKSTREAM = "NOMKSTREAM"
+    NOMKSTREAM = b"NOMKSTREAM"
 
     #: Used by:
     #:
     #:  - ``XAUTOCLAIM``
     #:  - ``XCLAIM``
-    JUSTID = "JUSTID"
+    JUSTID = b"JUSTID"
 
     #: Used by:
     #:
     #:  - ``XGROUP CREATE``
-    MKSTREAM = "MKSTREAM"
+    MKSTREAM = b"MKSTREAM"
 
     #: Used by:
     #:
     #:  - ``XGROUP CREATE``
     #:  - ``XGROUP SETID``
-    NEW_ID = "$"
+    NEW_ID = b"$"
 
     #: Used by:
     #:
     #:  - ``XREADGROUP``
-    NOACK = "NOACK"
+    NOACK = b"NOACK"
 
     #: Used by:
     #:
     #:  - ``ZADD``
-    INCREMENT = "INCR"
+    INCREMENT = b"INCR"
 
     #: Used by:
     #:
@@ -562,7 +564,7 @@ class PureToken(str, enum.Enum):
     #:  - ``ZREVRANGE``
     #:  - ``ZREVRANGEBYSCORE``
     #:  - ``ZUNION``
-    WITHSCORES = "WITHSCORES"
+    WITHSCORES = b"WITHSCORES"
 
     #: Used by:
     #:
@@ -570,22 +572,401 @@ class PureToken(str, enum.Enum):
     #:  - ``ZINTERSTORE``
     #:  - ``ZUNION``
     #:  - ``ZUNIONSTORE``
-    SUM = "SUM"
+    SUM = b"SUM"
 
     #: Used by:
     #:
     #:  - ``ZRANGE``
     #:  - ``ZRANGESTORE``
-    BYLEX = "BYLEX"
+    BYLEX = b"BYLEX"
 
     #: Used by:
     #:
     #:  - ``ZRANGE``
     #:  - ``ZRANGESTORE``
-    BYSCORE = "BYSCORE"
+    BYSCORE = b"BYSCORE"
 
     #: Used by:
     #:
     #:  - ``ZRANGE``
     #:  - ``ZRANGESTORE``
-    REV = "REV"
+    REV = b"REV"
+
+
+class PrefixToken(CaseAndEncodingInsensitiveEnum):
+    """
+    Enum for internal use when adding prefixes to arguments
+    """
+
+    #: Used by:
+    #:
+    #:  - ``BITFIELD``
+    #:  - ``BITFIELD_RO``
+    #:  - ``SORT``
+    #:  - ``SORT_RO``
+    GET = b"GET"
+
+    #: Used by:
+    #:
+    #:  - ``BITFIELD``
+    INCRBY = b"INCRBY"
+
+    #: Used by:
+    #:
+    #:  - ``BITFIELD``
+    OVERFLOW = b"OVERFLOW"
+
+    #: Used by:
+    #:
+    #:  - ``BITFIELD``
+    SET = b"SET"
+
+    #: Used by:
+    #:
+    #:  - ``BLMPOP``
+    #:  - ``BZMPOP``
+    #:  - ``GEORADIUS``
+    #:  - ``GEORADIUSBYMEMBER``
+    #:  - ``GEORADIUSBYMEMBER_RO``
+    #:  - ``GEORADIUS_RO``
+    #:  - ``GEOSEARCH``
+    #:  - ``GEOSEARCHSTORE``
+    #:  - ``HSCAN``
+    #:  - ``LMPOP``
+    #:  - ``LPOS``
+    #:  - ``SCAN``
+    #:  - ``SSCAN``
+    #:  - ``XAUTOCLAIM``
+    #:  - ``XINFO STREAM``
+    #:  - ``XRANGE``
+    #:  - ``XREAD``
+    #:  - ``XREADGROUP``
+    #:  - ``XREVRANGE``
+    #:  - ``ZMPOP``
+    #:  - ``ZSCAN``
+    COUNT = b"COUNT"
+
+    #: Used by:
+    #:
+    #:  - ``CLIENT KILL``
+    ADDR = b"ADDR"
+
+    #: Used by:
+    #:
+    #:  - ``CLIENT KILL``
+    #:  - ``CLIENT LIST``
+    ID = b"ID"
+
+    #: Used by:
+    #:
+    #:  - ``CLIENT KILL``
+    LADDR = b"LADDR"
+
+    #: Used by:
+    #:
+    #:  - ``CLIENT KILL``
+    SKIPME = b"SKIPME"
+
+    #: Used by:
+    #:
+    #:  - ``CLIENT KILL``
+    #:  - ``CLIENT LIST``
+    #:  - ``SCAN``
+    TYPE = b"TYPE"
+
+    #: Used by:
+    #:
+    #:  - ``CLIENT KILL``
+    USER = b"USER"
+
+    #: Used by:
+    #:
+    #:  - ``CLIENT TRACKING``
+    PREFIX = b"PREFIX"
+
+    #: Used by:
+    #:
+    #:  - ``CLIENT TRACKING``
+    REDIRECT = b"REDIRECT"
+
+    #: Used by:
+    #:
+    #:  - ``CLUSTER SETSLOT``
+    IMPORTING = b"IMPORTING"
+
+    #: Used by:
+    #:
+    #:  - ``CLUSTER SETSLOT``
+    MIGRATING = b"MIGRATING"
+
+    #: Used by:
+    #:
+    #:  - ``CLUSTER SETSLOT``
+    NODE = b"NODE"
+
+    #: Used by:
+    #:
+    #:  - ``COMMAND LIST``
+    ACLCAT = b"ACLCAT"
+
+    #: Used by:
+    #:
+    #:  - ``COMMAND LIST``
+    FILTERBY = b"FILTERBY"
+
+    #: Used by:
+    #:
+    #:  - ``COMMAND LIST``
+    MODULE = b"MODULE"
+
+    #: Used by:
+    #:
+    #:  - ``COMMAND LIST``
+    PATTERN = b"PATTERN"
+
+    #: Used by:
+    #:
+    #:  - ``COPY``
+    DB = b"DB"
+
+    #: Used by:
+    #:
+    #:  - ``FAILOVER``
+    TIMEOUT = b"TIMEOUT"
+
+    #: Used by:
+    #:
+    #:  - ``FAILOVER``
+    TO = b"TO"
+
+    #: Used by:
+    #:
+    #:  - ``FUNCTION LIST``
+    LIBRARYNAME = b"LIBRARYNAME"
+
+    #: Used by:
+    #:
+    #:  - ``GEORADIUS``
+    #:  - ``GEORADIUSBYMEMBER``
+    #:  - ``SORT``
+    STORE = b"STORE"
+
+    #: Used by:
+    #:
+    #:  - ``GEORADIUS``
+    #:  - ``GEORADIUSBYMEMBER``
+    STOREDIST = b"STOREDIST"
+
+    #: Used by:
+    #:
+    #:  - ``GEOSEARCH``
+    #:  - ``GEOSEARCHSTORE``
+    BYBOX = b"BYBOX"
+
+    #: Used by:
+    #:
+    #:  - ``GEOSEARCH``
+    #:  - ``GEOSEARCHSTORE``
+    BYRADIUS = b"BYRADIUS"
+
+    #: Used by:
+    #:
+    #:  - ``GEOSEARCH``
+    #:  - ``GEOSEARCHSTORE``
+    FROMLONLAT = b"FROMLONLAT"
+
+    #: Used by:
+    #:
+    #:  - ``GEOSEARCH``
+    #:  - ``GEOSEARCHSTORE``
+    FROMMEMBER = b"FROMMEMBER"
+
+    #: Used by:
+    #:
+    #:  - ``GETEX``
+    #:  - ``SET``
+    EX = b"EX"
+
+    #: Used by:
+    #:
+    #:  - ``GETEX``
+    #:  - ``SET``
+    EXAT = b"EXAT"
+
+    #: Used by:
+    #:
+    #:  - ``GETEX``
+    #:  - ``SET``
+    PX = b"PX"
+
+    #: Used by:
+    #:
+    #:  - ``GETEX``
+    #:  - ``SET``
+    PXAT = b"PXAT"
+
+    #: Used by:
+    #:
+    #:  - ``HELLO``
+    #:  - ``MIGRATE``
+    AUTH = b"AUTH"
+
+    #: Used by:
+    #:
+    #:  - ``HELLO``
+    SETNAME = b"SETNAME"
+
+    #: Used by:
+    #:
+    #:  - ``HSCAN``
+    #:  - ``SCAN``
+    #:  - ``SSCAN``
+    #:  - ``ZSCAN``
+    MATCH = b"MATCH"
+
+    #: Used by:
+    #:
+    #:  - ``LCS``
+    MINMATCHLEN = b"MINMATCHLEN"
+
+    #: Used by:
+    #:
+    #:  - ``LOLWUT``
+    VERSION = b"VERSION"
+
+    #: Used by:
+    #:
+    #:  - ``LPOS``
+    MAXLEN = b"MAXLEN"
+
+    #: Used by:
+    #:
+    #:  - ``LPOS``
+    RANK = b"RANK"
+
+    #: Used by:
+    #:
+    #:  - ``MEMORY USAGE``
+    SAMPLES = b"SAMPLES"
+
+    #: Used by:
+    #:
+    #:  - ``MIGRATE``
+    AUTH2 = b"AUTH2"
+
+    #: Used by:
+    #:
+    #:  - ``MIGRATE``
+    KEYS = b"KEYS"
+
+    #: Used by:
+    #:
+    #:  - ``MODULE LOADEX``
+    ARGS = b"ARGS"
+
+    #: Used by:
+    #:
+    #:  - ``MODULE LOADEX``
+    CONFIG = b"CONFIG"
+
+    #: Used by:
+    #:
+    #:  - ``RESTORE``
+    FREQ = b"FREQ"
+
+    #: Used by:
+    #:
+    #:  - ``RESTORE``
+    IDLETIME = b"IDLETIME"
+
+    #: Used by:
+    #:
+    #:  - ``SINTERCARD``
+    #:  - ``SORT``
+    #:  - ``SORT_RO``
+    #:  - ``XADD``
+    #:  - ``XTRIM``
+    #:  - ``ZINTERCARD``
+    #:  - ``ZRANGE``
+    #:  - ``ZRANGEBYLEX``
+    #:  - ``ZRANGEBYSCORE``
+    #:  - ``ZRANGESTORE``
+    #:  - ``ZREVRANGEBYLEX``
+    #:  - ``ZREVRANGEBYSCORE``
+    LIMIT = b"LIMIT"
+
+    #: Used by:
+    #:
+    #:  - ``SORT``
+    #:  - ``SORT_RO``
+    BY = b"BY"
+
+    #: Used by:
+    #:
+    #:  - ``XCLAIM``
+    #:  - ``XPENDING``
+    IDLE = b"IDLE"
+
+    #: Used by:
+    #:
+    #:  - ``XCLAIM``
+    RETRYCOUNT = b"RETRYCOUNT"
+
+    #: Used by:
+    #:
+    #:  - ``XCLAIM``
+    TIME = b"TIME"
+
+    #: Used by:
+    #:
+    #:  - ``XGROUP CREATE``
+    #:  - ``XGROUP SETID``
+    ENTRIESREAD = b"ENTRIESREAD"
+
+    #: Used by:
+    #:
+    #:  - ``XINFO STREAM``
+    FULL = b"FULL"
+
+    #: Used by:
+    #:
+    #:  - ``XREAD``
+    #:  - ``XREADGROUP``
+    BLOCK = b"BLOCK"
+
+    #: Used by:
+    #:
+    #:  - ``XREAD``
+    #:  - ``XREADGROUP``
+    STREAMS = b"STREAMS"
+
+    #: Used by:
+    #:
+    #:  - ``XREADGROUP``
+    GROUP = b"GROUP"
+
+    #: Used by:
+    #:
+    #:  - ``XSETID``
+    ENTRIESADDED = b"ENTRIESADDED"
+
+    #: Used by:
+    #:
+    #:  - ``XSETID``
+    MAXDELETEDID = b"MAXDELETEDID"
+
+    #: Used by:
+    #:
+    #:  - ``ZINTER``
+    #:  - ``ZINTERSTORE``
+    #:  - ``ZUNION``
+    #:  - ``ZUNIONSTORE``
+    AGGREGATE = b"AGGREGATE"
+
+    #: Used by:
+    #:
+    #:  - ``ZINTER``
+    #:  - ``ZINTERSTORE``
+    #:  - ``ZUNION``
+    #:  - ``ZUNIONSTORE``
+    WEIGHTS = b"WEIGHTS"
